
#include "bldpath.h"

#include <stdio.h>

#define DEFAULT_DOCUMENT_NAME	(L"default.htm")


// /////////////////////////////////////////////////////////////////////////////
//
// pLocalPath -- L"C:\\Inetpub\\wwwroot"
// pURL          -- must start with L'/'. L"/", L"/vdir", L"/vdir/, L"/foo.htm" 
//

BOOL GetLocalPathFromURL ( PCWSTR pszRootPath, PCWSTR pszURL, LPWSTR pLocalPath, ULONG cbLocalPath ) {
	
	BOOL		bRet = FALSE;
	WCHAR		pBuffer [MAX_PATH];

	// Validate input
	if ( !pszRootPath || !pszURL ) {
		SetLastError ( ERROR_INVALID_PARAMETER );
		return bRet;
	}

	// Length
	if ( wcslen (pszRootPath) + wcslen (pszURL) > MAX_PATH ) {
		SetLastError ( ERROR_INVALID_PARAMETER );
		return bRet;
	}

	// Copy the data to a local buffer.
	// because we may need to write the buffer.
	wcscpy ( pBuffer, pszRootPath );

	// Check not to terminate the root path with L'\\'
	if ( pBuffer [wcslen (pBuffer) - 1] == L'\\' ) {
		pBuffer [wcslen (pBuffer) - 1] = L'\0';
	}

	// Build Path
	wcscat ( pBuffer, pszURL );

	// Convert '/' to '\\' in relative path.
	for ( int i=wcslen (pszURL)-1; i<wcslen(pBuffer); i++) {
		if ( L'/' == pBuffer[i] ) {
			pBuffer[i] = L'\\';
		}
	}

	// Set data into the return value.
	if ( cbLocalPath > ( (wcslen (pBuffer)+1) * sizeof (WCHAR)) ) {
		wcscpy (pLocalPath, pBuffer);
		bRet = TRUE; // Success
	}
	else {
		// Buffer is too small.
		SetLastError ( ERROR_INSUFFICIENT_BUFFER );
	}

	return bRet;

}


///////////////////////////////////////////////////////////////////////////////


HANDLE GetFileHandleFromURL ( PCWSTR pszRootPath, PCWSTR pszURL, LPWSTR pOutFileName, ULONG cbOutFileName ) {

	HANDLE hFile = INVALID_HANDLE_VALUE;

	// Validate input (NULL)
	if (!pszRootPath || !pszURL || (cbOutFileName < MAX_PATH * sizeof (WCHAR)) ) {
		SetLastError ( ERROR_INVALID_PARAMETER );
		return hFile;
	}

	//
	// Get file name
	//

	// Build local path
	WCHAR pszFileName [MAX_PATH];

	if (!GetLocalPathFromURL ( pszRootPath, pszURL, pszFileName, MAX_PATH * sizeof (WCHAR) ) ) {
		return hFile;
	}

	// If the local path ends with '\\', it's apparently directory.
	// So set the default document. 
	if ( pszFileName[wcslen (pszFileName) -1] == L'\\' ) {
		if ( wcslen ( pszFileName ) + wcslen ( DEFAULT_DOCUMENT_NAME ) > MAX_PATH ) {
			return hFile;
		}

		wcscat ( pszFileName, DEFAULT_DOCUMENT_NAME );
	}

	// Open the file.
	hFile = CreateFileW (
		pszFileName,
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL);

	if ( INVALID_HANDLE_VALUE != hFile ) {
		wcscpy ( pOutFileName, pszFileName );
		return hFile;
	}

	wprintf ( L"CreateFileW failed with %u - %s\n", GetLastError(), pszFileName);

	// If it failed try again with the default document.

	if ( wcslen ( pszFileName ) + wcslen ( DEFAULT_DOCUMENT_NAME ) + 1 > MAX_PATH ) {
		return hFile;
	}

	wcscat ( pszFileName, L"\\" );
	wcscat ( pszFileName, DEFAULT_DOCUMENT_NAME );

	hFile = CreateFileW (
		pszFileName,
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL);

	if ( INVALID_HANDLE_VALUE != hFile ) {
		wcscpy ( pOutFileName, pszFileName );
	}

	return hFile; 

}


///////////////////////////////////////////////////////////////////////////////


BOOL GetContentType ( LPWSTR pszFilename, PCHAR pContentType, PULONG pcbContentType) {

	BOOL bRet = FALSE;

	CONTENT_TYPE ContentTypes [] = {
		{L".htm", "text/html"},
		{L".html", "text/html"},
		{L".gif", "image/gif"},
		{L".jpeg", "image/jpeg"},
		{L".jpg", "image/jpeg"},
		{L".bmp", "image/bmp"},
		{L".zip", "application/zip"}
	};

	for (int i=0; i < sizeof (ContentTypes)/sizeof (ContentTypes[0]); i++) {

		if ( EndWith ( pszFilename, ContentTypes[i].pszExt ) ) {
			if ( strlen (ContentTypes[i].pszContentType) < *pcbContentType ) {
				strcpy ( pContentType, ContentTypes[i].pszContentType );
				*pcbContentType = strlen (ContentTypes[i].pszContentType);
				bRet = TRUE;
			}
			else {
				SetLastError ( ERROR_INSUFFICIENT_BUFFER );
			}
			break;
		}

	}

	return bRet;
}


///////////////////////////////////////////////////////////////////////////////


BOOL EndWith ( PCWSTR pszFilename, PCWSTR pszExt ) {

	BOOL bRet = FALSE;

	// Validate input
	if ( !pszFilename || !pszExt ) {
		SetLastError ( ERROR_INVALID_PARAMETER );
		return bRet;
	}

	if ( wcslen ( pszFilename) <= wcslen (pszExt) ) {
		SetLastError ( ERROR_INVALID_PARAMETER );
		return bRet;
	}

	// 
	PCWSTR p = pszFilename;
	p += wcslen ( pszFilename );
	p -= wcslen ( pszExt );

	if ( !wcsicmp ( p, pszExt ) ) {
		bRet = TRUE;
	}

	return bRet;
}